
texture DiffuseTexture : TEXTURE0
<
	string ResourceType = "2D";
>;
 
sampler2D DiffuseTextureSampler = sampler_state
{
	Texture = <DiffuseTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};
 
float4 SpecularColor : CHANNELVECTOR0
<
	string UIName = "Specular Color";
	string UIWidget = "Color";
> = {0.0f, 0.0f, 0.65098f, 1.0f };
 
float Glossiness :CHANNELVALUE0
<
	string UIWidget = "Slider";
	float UIMin = 0.0;
	float UIMax = 10.0;
	float UIStep = 0.1;
	string UIName = "Glossiness";
> = 11.0;
 
texture NormalMap : TEXTURE1
<
	string ResourceType = "2D";
>;
 
sampler2D NormalMapSampler = sampler_state
{
	Texture = <NormalMap>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};




// ViewProjection
float4x4 vp:ViewProjection;

// NaturePainter fills these arrays
// This method of rendering is mainly usefull for lot's of low poly items
static const int MAX_MATRICES = 40; // Max matrices per batch
float4x4 worldArray[MAX_MATRICES] : MATRIXARRAY; // World Matrix Array
float    fadeArray[MAX_MATRICES] : ARRAYVALUE; // FadeValue Array

float4x4 wvp : WorldViewProjection < string UIWidget = "None"; >;  
float4x4 worldI : WorldInverse < string UIWidget = "None"; >;  
float4x4 worldIT : WorldInverseTranspose < string UIWidget = "None"; >;  
float4x4 viewInv : ViewInverse < string UIWidget = "None"; >;  
float4x4 world : World < string UIWidget = "None"; >;  




float3 light_pos[8]	: LIGHTPOS; 
 
float4 light_color[8]	: LIGHTCOLOR; 

// VS.IN
struct VS_INPUT
{
	float4 position	: POSITION;
	float2 texCoord		: TEXCOORD0;
	float  index 		: TEXCOORD1;
	float3 tangent		: TANGENT; 
	float3 binormal		: BINORMAL; 
	float3 normal		: NORMAL; 


};

// VS.OUT
struct VS_OUTPUT
{
	float4 position	: POSITION;
	float  fade : TEXCOORD0;
	float2 texCoord		: TEXCOORD2;
    float3 light1Vec    	: TEXCOORD3; 
    float3 eyeVec	    	: TEXCOORD4; 


};



// Vertex Shader
VS_OUTPUT VS_NaturePaintExample(VS_INPUT In,  uniform float3 lightPosition)
{
	VS_OUTPUT Out;
	float4x4 w = worldArray[In.index]; // World from naturepainter
	float4 wpos = mul(In.position, w); // Construct a new World matrix for each instance
	Out.position = mul(wpos, vp); // multiply with ViewProjection to get WorldViewProjection
	Out.fade = fadeArray[In.index];


	float3x3 objTangentXf;								//build object to tangent space transform matrix 
    	objTangentXf[0] = In.tangent; 
    	objTangentXf[1] = -In.binormal; 
    	objTangentXf[2] = In.normal; 
    //this code was added by the standard material 
    float3 wsL1Pos = mul(In.position, world).xyz;			//put the vert position in world space 
    float3 wsL1Vec = light_pos[0] - wsL1Pos;    //cast a ray to the light 
    float3 osL1Vec = mul(wsL1Vec, worldI).xyz;  //transform the light vector to object space 
    Out.light1Vec = mul(objTangentXf, osL1Vec);			//tangent space light vector passed out 
    //this code was added by the standard material 
    float4 osIPos = mul(viewInv[3], worldI);			//put world space eye position in object space 
    float3 osIVec = osIPos.xyz - In.position.xyz;		//object space eye vector 
    Out.eyeVec = mul(objTangentXf, osIVec);		

	Out.texCoord = In.texCoord;						//pass through texture coordinates from channel 1 
	return Out;



};


// Pixel Shader
float4 PS_NaturePaintExample(VS_OUTPUT In, uniform float4 lightColor) : COLOR
{
//	return float4(0.2,0.5,1.0, In.fade);
	float3 ret =  float3(0,0,0); 
	float4 DiffuseTexture = tex2D(DiffuseTextureSampler, In.texCoord.xy);
	float3 input2 = DiffuseTexture.rgb; 

	float4 NormalMap = tex2D(NormalMapSampler, In.texCoord.xy);
	NormalMap.xyz = NormalMap.xyz * 2 - 1;		//expand to -1 to 1 range 
	NormalMap.rgb = normalize(NormalMap.rgb); 		//normalized the normal vector 
	float3 input8 = NormalMap.rgb; 


	float3 V = normalize(In.eyeVec);
	float3 N = input8;						//using the Normal socket  
	float3 spec = float3(0,0,0); 
	float3 specularColor = SpecularColor;			//using the Specular Color socket 
	float glossiness = Glossiness;		

	float3 L1 = normalize(In.light1Vec.xyz); 
	float3 light1 = saturate(dot(N, L1)) * light_color[0].xyz  * input2;		//diffuse light formula for light1  
	ret +=  light1; 
	float3 H1 = normalize(L1 + V); 
	float NdotH1 = saturate(dot(N,H1));		//Compute NdotH1  
	ret += pow(NdotH1, glossiness) * light_color[0] * specularColor;//Raise to glossiness power and compute final specular color  

	ret +=input2;
	float4 done = float4(ret, In.fade); 
	return done; 


	
};

// Technique
technique NaturePaintExample
{

	pass p0
	{
		VertexShader = compile vs_3_0 VS_NaturePaintExample(light_pos[0]);
		PixelShader = compile ps_3_0 PS_NaturePaintExample(light_color[0]);
	}
};