/******************************************************************************************************
Heat-Haze-Shader by Wolfgang "BoH_Havoc" Reichardt

Entity Textures:
	Skin1 = Normalmap
	Skin2 = Colormap 

Usage:
	Uncomment/Comment the #defines to add/remove an effect.
	vecSkill1.y sets the movement of the normalmap
	
******************************************************************************************************/


/***************************************TWEAKABLES*****************************************************/

//Refraction Strength
float refractStrength = 0.15;

//Use Colormap (entSkin2) to color the entity
#define USECOLOR

/***************************************SHADER*CODE****************************************************/

float4x4 matWorldViewProj;
float4x4 matViewInv;
float4x4 matWorld;

float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill17;
float4 vecTime;
float4 vecViewPos;
float4 vecViewDir;

texture entSkin1;

sampler normalmapSampler = sampler_state
{
	Texture = <entSkin1>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;   
	//MinFilter = POINT;
	//MagFilter = POINT;
	//MipFilter = POINT;   
	AddressU  = wrap;
	AddressV  = wrap;
	
};

texture entSkin2;

sampler colorSampler = sampler_state
{
	Texture = <entSkin2>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;   
	//MinFilter = POINT;
	//MagFilter = POINT;
	//MipFilter = POINT;   
	AddressU  = wrap;
	AddressV  = wrap;
	
};

texture entSkin3;

sampler lightningSampler = sampler_state
{
	Texture = <entSkin3>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;   
	//MinFilter = POINT;
	//MagFilter = POINT;
	//MipFilter = POINT;   
	AddressU  = wrap;
	AddressV  = wrap;
	
};

texture sc_map_refract_bmap;

sampler refractionSampler=sampler_state
{
	texture=<sc_map_refract_bmap>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    addressu=clamp;
    addressv=clamp;
};

texture sc_map_scene_bmap;

sampler sceneSampler=sampler_state
{
	texture=<sc_map_scene_bmap>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    addressu=clamp;
    addressv=clamp;
};

void heat_VS( in float4 inPos : POSITION, //input vertex position
					in float2 inTex : TEXCOORD0, //input texcoords
					in float3 inNormal : NORMAL,
				   out float4 outPos : POSITION, //ouput transformed position
				   out float4 outTex : TEXCOORD0, //output texcoords
				   out float4 outProjectedTex : TEXCOORD1, //output projected texcoords
				   out float4 outWPos : TEXCOORD2, //output worldpos
				   out float4 outNormal : TEXCOORD3 //output normal
				   )
{
	
	outPos=mul(inPos,matWorldViewProj);
	//save the vertex information for further projected texture coordinate computation
	outProjectedTex=outPos;
	
	float4 ShadowMapSamplingPos = mul(inPos, matWorldViewProj);

	outProjectedTex.x = ShadowMapSamplingPos.x;
	outProjectedTex.y = ShadowMapSamplingPos.y;
	outProjectedTex.z = ShadowMapSamplingPos.w;
	
	//outProjectedTex = ShadowMapSamplingPos;
	
	//ouput texcoords
	outTex.xy=inTex+(vecTime.w*0.001*vecSkill1.y);
	outTex.zw=inTex;
	
	outWPos = mul(inPos,matWorld);
	outNormal = mul(inNormal,matWorld);
}


float4 heat_PS(in float4 inTex:TEXCOORD0, //input texture coords
					in float4 inProjectedTex:TEXCOORD1, //projectedTexcoords, needed to perform reflections,refractions
					in float4 inWPos : TEXCOORD2,
					in float3 inNormal : TEXCOORD3
):COLOR0
{
	clip(vecSkill1.x);
	
	float2 ProjectedTexCoords;
   ProjectedTexCoords[0] = inProjectedTex.x/inProjectedTex.z/2.0f +0.5f;
   ProjectedTexCoords[1] = -inProjectedTex.y/inProjectedTex.z/2.0f +0.5f;
   
	float3 pixelNormal=tex2D(normalmapSampler,inTex.xy).xyz;
	//float3 pixelNormal=tex2D(normalmapSampler,ProjectedTexCoords.xy);
   pixelNormal.yz=pixelNormal.zy;
   pixelNormal.xyz=(pixelNormal.xyz-0.5f)*2;
   
   
   
   
   
   float4 refraction=tex2D(refractionSampler,ProjectedTexCoords.xy-refractStrength*pixelNormal.xz);
	float4 scene=tex2D(sceneSampler,ProjectedTexCoords.xy);
	
	float4 color = 1;
	
	
	
	float3 Nn = normalize(inNormal);
	float3 Vn = normalize(matViewInv[3].xyz - inWPos.xyz);
		
	half3 velvety1;
   float vdn = 1.5-dot(Vn,Nn);
   float3 vecColor = float4(vdn.xxx,1.0);
   velvety1 = float4((vecColor).xyz,1);
   velvety1 = clamp(velvety1,0,1);
   
   half3 velvety2;
   vdn = 1.0-dot(Vn,Nn);
   vecColor = float4(vdn.xxx,1.0);
   velvety2 = float4((vecColor).xyz,1);
   velvety2 = clamp(velvety2,0,1);
   //velvety2 = pow(velvety2,2);
   
   float4 alphaNoise = tex2D(colorSampler,inTex.zw-refractStrength*pixelNormal.xz);
   //float alphaNoise2 = tex2D(colorSampler,inTex.zw-refractStrength*pixelNormal.xz).g;
   //float alphaNoise3 = tex2D(colorSampler,inTex.zw-refractStrength*pixelNormal.xz).b;
   alphaNoise.r = pow(alphaNoise.r,2);
   
   
	color.a = 1;//tex2D(colorSampler,inTex.zw).rgb;
	color.rgb = refraction;//lerp(scene,refraction,alphaNoise.r);
	color.rgb -= (alphaNoise.g*velvety1);
	color.rgb += (alphaNoise.r-alphaNoise.g)*float3(0,1,2);
	//color.rgb += alphaNoise3*(1-velvety2);
	
	
	color.rgb += alphaNoise.b*float3(1,1,1);
	color.rgb = clamp(color.rgb,0,1);
	
	
	color.rgb += alphaNoise.b*float3(1,2,4);
	//color.rgb += alphaNoise.b*float3(1,1,2);
	color.rgb = clamp(color.rgb,0,1);
	
	//lightning
	float4 lightning = tex2D(lightningSampler,(inTex.zw*vecSkill17.x)+float2(vecSkill17.x,vecSkill17.x)-refractStrength*pixelNormal.xz);
	//float4 lightning = tex2D(lightningSampler,inTex.zw+float2(vecSkill17.x,vecSkill17.x));
	//lightning += tex2D(lightningSampler,(inTex.zw*vecSkill17.x)+float2(vecSkill17.x,vecSkill17.x));
	float4 lightningS = tex2D(lightningSampler,(inTex.zw*vecSkill17.x/2)+float2(vecSkill17.x,vecSkill17.x)-refractStrength*pixelNormal.xz);
	//lightning.a *= alphaNoise.g;
	lightning.a *= pow(alphaNoise.r,1.5);
	lightning.rgb *= lightning.a;
	//lightningS.a *= pow(1-alphaNoise.r,3.5)*velvety2*(1-velvety1)*(1-velvety2);
	lightningS.a *= (alphaNoise.r - alphaNoise.g);
	lightningS.rgb *= lightningS.a;
	color.rgb += lightning.rgb;
	//
	color.a = alphaNoise.r;
	//color.rgb = refraction;
	
	//color.rgb = alphaNoise2
	
	//color.rgb = ;

	
   return color;
};

technique heat
{

	pass p0
	{
		alphablendenable = true;
		zwriteenable = true;
		CULLMODE = CCW;
		VertexShader  = compile vs_2_0 heat_VS();
		PixelShader = compile ps_2_0 heat_PS();
	}
}