float4x4 matWorld;
float4x4 matWorldInv;
float4x4 matMtl;
float4x4 matView;
float4x4 matProj;
float4x4 matViewInv;
float4x4 matWorldViewProj;

//color of the light 
float4 vecLightColor[8];

//position of the camera, for specular light
//float3 cameraPosition; 
float4 vecViewPos;
float4 vecViewPort;
float4 vecViewDir;

//this is the position of the light
float4 vecLightPos[8];
float4 vecSunDir;

//control the brightness of the light
float lightIntensity = 1.0f;


// diffuse color, and specularIntensity in the alpha channel
texture mtlSkin1; 
// normals, and specularPower in the alpha channel
texture mtlSkin2;
//depth
texture mtlSkin3;
//Environment Map
texture mtlSkin4;

sampler colorSampler = sampler_state
{
    Texture = (mtlSkin1);
    //AddressU = WRAP;
    //AddressV = WRAP;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
};
sampler depthSampler = sampler_state
{
    Texture = (mtlSkin3);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = POINT;
    MinFilter = POINT;
    Mipfilter = POINT;
};
sampler normalSampler = sampler_state
{
    Texture = (mtlSkin2);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = POINT;
    MinFilter = POINT;
    Mipfilter = POINT;
};

sampler EnvMapSampler = sampler_state
{
	Texture = (mtlSkin4);
    //AddressU = WRAP;
    //AddressV = WRAP;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
};

sampler NoiseSampler = sampler_state
{
	Texture = (mtlSkin4);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = POINT;
    MinFilter = POINT;
    Mipfilter = POINT;
};

float unpack(float3 v, float maxVal)
{
	float3 vtf = float3(1.0 , 1.0 / maxVal, 1.0 / (maxVal*maxVal));
	return dot(v,vtf);
}


//uniform float3 ranSphere[32];
uniform float3 ranSphere[14] =
{
	1,0,0,
	0,1,0,
	0,0,1,
	
	-1,0,0,
	0,-1,0,
	0,0,-1,
	
	-1,-1,-1,
	1,1,1,
	
	-1,-1,0,
	0,-1,-1,
	-1,0,-1,
	
	1,1,0,
	0,1,1,
	1,0,1,
};

float4 PixelShaderFunction(float2 InTex : TEXCOORD0) : COLOR0
//float4 PixelShaderFunction(float2 InTex:TEXCOORD0) : COLOR0
{
    
	 InTex.xy *= 1;
    //get normal data from the normalMap
    float4 normalData = tex2D(normalSampler,InTex);
    //tranform normal back into [-1,1] range
    float3 normal = 2.0f * normalData.xyz - 1.0f;
    //get specular power

    //read depth
    float depth = tex2D(depthSampler,InTex).r;
	 //float4 vp = float4(InTex.xy * float2(2, -2) - float2(1, -1), depth, 1);
    //float4 v = mul(vp, matMtl);
    //float3 position = v.xyz/v.w;


	//float3 normals = normalize( cross(ddx(position.xyz),ddy(position.xyz)) );
	//normals = lerp(normals,normal,0.5);
	//float4 noise = texCUBE(NoiseSampler,float4(normal,1));
	float4 noise = tex2D(NoiseSampler,InTex*160);
	float3 ranSample;
	float rayDepth = 0.12;
	float rayDepth2 = 0.1;
	float ssao = 0;
	float ssao2 = 0;
	float ssaoBck = 0;
	float depthInv = 1-depth;
	float3 color = 0;
	
	for(int i=0; i<6; i++)
	{
	// Get the random vector inside the sphere
		ranSample = reflect( (ranSphere[i]) *40*depthInv, noise.xyz);
		//ranSample = reflect(normalize(normals)*ranSphere[i]*1.8*depth, noise.xyz);

		// Dither the length of the sample ray
		//ranSample *= noise.w;

		// Flip the sample if it's pointing away from the surface normal
		//if(dot(ranSample, normal) < 0.0) ranSample = -ranSample;
		//ranSample += 0.02 * normal;	// Bias towards the normal, to reduce self occlusion
		
		ranSample = ranSample*0.01*(depth);
		
		float depthS = tex2D(depthSampler,float2(InTex.xy)+ranSample.xy).r;
		float3 normals2 = tex2D(normalSampler,float2(InTex.xy)+ranSample.xy).rgb;
		float3 colorS = tex2D(colorSampler,float2(InTex.xy)+ranSample.xy).rgb;
		normals2 = 2.0f * normals2 - 1.0f;
		//if(depthS - depth < rayDepth && depthS - depth > -rayDepth && (depthS - depth > 0.0001 || depthS - depth < -0.0001)) ssao += 1;//-(depthS - depth);
		
		//if(depthS - depth < rayDepth) ssao += 1;
		//if( (1-depthS) - (1-depth) > rayDepth || (1-depthS) - (1-depth) < -rayDepth) ssao += 1;
		//if(depthS - depth < rayDepth) ssao += depth - depthS;
		
		if(
		depthS - depth < 0 && depthS - depth > -rayDepth*(depthInv)
			&& ((normal.r - normals2.r > rayDepth2 || normal.g - normals2.g > rayDepth2 || normal.b - normals2.b > rayDepth2)
			|| (normal.r - normals2.r < -rayDepth2 || normal.g - normals2.g < -rayDepth2 || normal.b - normals2.b < -rayDepth2))
		) 
		{
			ssao +=  1;//depthS;
			//color += colorS;
		}
		
		if(
		depthS - depth > 0 && depthS - depth < rayDepth*(depthInv)
		&& ((normal.r - normals2.r > rayDepth2 || normal.g - normals2.g > rayDepth2 || normal.b - normals2.b > rayDepth2)
			|| (normal.r - normals2.r < -rayDepth2 || normal.g - normals2.g < -rayDepth2 || normal.b - normals2.b < -rayDepth2))
		)
		{
			ssao2 +=  1;//depthS;
			//color += colorS;
		} 
		//else ssao -= 1-depth;
	}

	ssao /= 6;
	ssao2 /= 6;
	ssao = (1-ssao)+(ssao2);//*100;
	//color /= 6;
	//ssao = clamp(ssao,0,1);
	//ssao2 = clamp(ssao2,0,1);
	
	//normals = reflect(normalize(ranSample),normalize(normals));
	
	//return float4(color,ssao);
	
	return float4(ssao,0,0,1);// * float4(tex2D(colorSampler, InTex).rgb,1);
}



technique Technique1
{
    pass Pass1
    {
        //VertexShader = compile vs_2_0 VertexShaderFunction();
        PixelShader = compile ps_2_a PixelShaderFunction();
    }
}
