float4x4 matWorld;
float4x4 matWorldInv;
float4x4 matMtl;
float4x4 matView;
float4x4 matProj;
float4x4 matViewInv;
float4x4 matWorldViewProj;

//color of the light 
float4 vecLightColor[8];

//position of the camera, for specular light
//float3 cameraPosition; 
float4 vecViewPos;
float4 vecViewPort;
float4 vecViewDir;

//this is the position of the light
float4 vecLightPos[8];
float4 vecSunDir;


// diffuse color, and specularIntensity in the alpha channel
texture mtlSkin1; 
// normals, and specularPower in the alpha channel
texture mtlSkin2;
//depth
texture mtlSkin3;
//Environment Map
texture mtlSkin4;

sampler colorSampler = sampler_state
{
    Texture = (mtlSkin1);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
};
sampler depthSampler = sampler_state
{
    Texture = (mtlSkin3);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = POINT;
    MinFilter = POINT;
    Mipfilter = POINT;
};
sampler normalSampler = sampler_state
{
    Texture = (mtlSkin2);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = POINT;
    MinFilter = POINT;
    Mipfilter = POINT;
};

sampler EnvMapSampler = sampler_state
{
	Texture = (mtlSkin4);
    AddressU = WRAP;
    AddressV = WRAP;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
    Mipfilter = LINEAR;
};

float4 PixelShaderFunction(float2 InTex : TEXCOORD0) : COLOR0
//float4 PixelShaderFunction(float2 InTex:TEXCOORD0) : COLOR0
{
    
	 
    //get normal data from the normalMap
    float4 normalData = tex2D(normalSampler,InTex);
    //tranform normal back into [-1,1] range
    float3 normal = 2.0f * normalData.xyz - 1.0f;
    //float3 normal = normalData.xyz;
    //get specular power
    float specularPower = normalData.a * 255;
    //get specular intensity from the colorMap
    float specularIntensity = tex2D(normalSampler, InTex).a;

    //read depth
    float depth = tex2D(depthSampler,InTex).r;
	 float4 vp = float4(InTex.xy * float2(2, -2) - float2(1, -1), depth, 1);
    float4 v = mul(vp, matMtl);
    float3 position = v.xyz/v.w;

    //camera-to-surface vector
    float3 directionToCamera = normalize(vecViewPos.xyz - position);


	//---- DIFFUSE+SPECULAR LIGHTING ----
 	
 	float3 Ln = (vecLightPos[0].xyz - position);
 	float attenuation = saturate(1.0f - length(Ln)/vecLightPos[0].w);
   Ln = normalize(Ln);
	float3 Nb = normalize(normal);
   float3 Vn = directionToCamera;
   float3 Hn = normalize(Vn + Ln);
   float4 lighting = lit(dot(Ln,Nb),dot(Hn,Nb),15);
   
   Ln = (vecLightPos[1].xyz - position);
   float attenuation2 = saturate(1.0f - length(Ln)/vecLightPos[1].w);
   Ln = normalize(Ln);
   Hn = normalize(Vn + Ln);
   float4 lighting2 = lit(dot(Ln,Nb),dot(Hn,Nb),15) * attenuation2;
     
   Ln = (vecLightPos[2].xyz - position);
   float attenuation3 = saturate(1.0f - length(Ln)/vecLightPos[2].w);
   Ln = normalize(Ln);
   Hn = normalize(Vn + Ln);
   float4 lighting3 = lit(dot(Ln,Nb),dot(Hn,Nb),15) * attenuation3;
   
   Ln = (vecLightPos[3].xyz - position);
   float attenuation4 = saturate(1.0f - length(Ln)/vecLightPos[3].w);
   Ln = normalize(Ln);
   Hn = normalize(Vn + Ln);
   float4 lighting4 = lit(dot(Ln,Nb),dot(Hn,Nb),15) * attenuation4;
   
   Ln = (vecLightPos[4].xyz - position);
   float attenuation5 = saturate(1.0f - length(Ln)/vecLightPos[4].w);
   Ln = normalize(Ln);
   Hn = normalize(Vn + Ln);
   float4 lighting5 = lit(dot(Ln,Nb),dot(Hn,Nb),15) * attenuation5;
   
   Ln = (vecLightPos[5].xyz - position);
   float attenuation6 = saturate(1.0f - length(Ln)/vecLightPos[5].w);
   Ln = normalize(Ln);
   Hn = normalize(Vn + Ln);
   float4 lighting6 = lit(dot(Ln,Nb),dot(Hn,Nb),15) * attenuation6;
   
   	
 	lighting.w = lighting.z;
 	lighting.xyz = (lighting.y+clamp((1-lighting.y)-0.9,0,1)) * vecLightColor[0].xyz;
 	
 	lighting2.w = lighting2.z;
 	lighting2.xyz = lighting2.y * vecLightColor[1].xyz;
 	
 	lighting3.w = lighting3.z;
 	lighting3.xyz = lighting3.y * vecLightColor[2].xyz;
 	
 	lighting4.w = lighting4.z;
 	lighting4.xyz = lighting4.y * vecLightColor[3].xyz;
 	
 	lighting5.w = lighting5.z;
 	lighting5.xyz = lighting5.y * vecLightColor[4].xyz;
 	
 	lighting6.w = lighting6.z;
 	lighting6.xyz = lighting6.y * vecLightColor[5].xyz;

	
	//---- ENVIRONMENT MAPPING ----
	Vn = normalize(matViewInv[3].xyz - position);
	float3 reflVect = reflect(Vn,normalize(normal));
	float3 reflColor = texCUBE(EnvMapSampler,float4(-reflVect, 1)).rgb * tex2D(colorSampler, InTex).a;
 	//
 	//---- ENVIRONMENT ILLUMINATION MAPPING ----
	float4 envLight = texCUBE(EnvMapSampler,float4(normal, 1));
	//envLight.xyz = lerp(envLight.xyz,reflColor.xyz,0.5);
	//
	//---- VELVETY ----
	Ln = (vecViewPos.xyz - position);
   Ln = normalize(Ln);
   Hn = normalize(Vn + Ln);
   float4 velvety = lit(dot(Ln,Nb),dot(Hn,Nb),2);
   velvety.y = 1-velvety.y;
   //
   
	float4 finalLighting = lighting;//float4(lighting + lighting2 + lighting3 + lighting4 + lighting5 + lighting6);
	//finalLighting.xyz *= 2;
	
	if(depth == 0) //skycube
	{
		finalLighting.rgb = 1;
		finalLighting.a = 0;
		envLight = 0;
		velvety = 0;
		reflColor = 0;
	}
	else{
		finalLighting.xyz += (envLight.xyz/2)*tex2D(colorSampler, InTex).a;//lerp(envLight.xyz,finalLighting.xyz,0.5);
	}
	
	//finalLighting.xyz = max(pow(finalLighting.xyz+0.1,2),finalLighting.xyz);
	//finalLighting.xyz *= 1.5;
	//return float4(finalLighting.xyz,1);
	//return float4(reflColor.xyz,1);
 	//return tex2D(EnvMapSampler,InTex).r * float4(tex2D(colorSampler, InTex).rgb,1);//finalLighting * float4(reflColor,1);
 	
 	
 	//color.rgb = ((envLight.xyz+(lighting.y*vecLightColor[0].xyz)) * texCol) + lighting.z*mask.r;
   //color.rgb += normal.a * lumColor;
 	
 	float4 color = 0;
 	color.rgb = finalLighting*tex2D(colorSampler, InTex).rgb + finalLighting.w * specularIntensity;
 	
 	return color;
 	
 	
 	
 	
 	
 	
 	return finalLighting * float4(tex2D(colorSampler, InTex).rgb,1)
 				 + finalLighting.w*specularIntensity
 				 + float4(reflColor,1) * float4(tex2D(colorSampler, InTex).rgb,1) * specularIntensity
 				 + velvety.y * (envLight/2) * float4(tex2D(colorSampler, InTex).rgb,1);
 				 //+ envLight * float4(tex2D(colorSampler, InTex).rgb,1);
 				 //+ finalLighting * float4(reflColor,1);  
}



technique Technique1
{
    pass Pass1
    {
        //VertexShader = compile vs_2_0 VertexShaderFunction();
        PixelShader = compile ps_2_a PixelShaderFunction();
    }
}
