float4x4 matMtl;
float4x4 matWorld;
float4x4 matWorldViewProj;

float4x4 matEffect1;
float4x4 matEffect2;
float4x4 matEffect3;
float4x4 matEffect4;

float4 vecViewPos;
float4 vecSunDir;
float4 vecSunColor;
float4 vecLightPos[8];
float4 vecLightColor[8];

texture TargetMap;

sampler sceneSampler = sampler_state 
{ 
   Texture = <TargetMap>; 
   MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
};

texture sc_map_depthEncode_bmap;

sampler depthSampler = sampler_state 
{ 
   Texture = <sc_map_depthEncode_bmap>; 
   MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	
};

// ShadowDepth map sampler
texture sc_map_shadowDepthSun_bmap;

sampler ShadowMapSampler = sampler_state
{
	Texture = <sc_map_shadowDepthSun_bmap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};

Texture sc_map_shadowAlpha_bmap;

sampler ShadowAlphaSampler = sampler_state
{
	texture = <sc_map_shadowAlpha_bmap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter=LINEAR;
	AddressU = clamp;
	AddressV = clamp;
};

texture sc_map_2Dnoise_bmap;

sampler noiseSampler = sampler_state
{
	texture = <sc_map_2Dnoise_bmap>;
	magfilter = LINEAR;
	minfilter = LINEAR;
	mipfilter=LINEAR;
};

texture sc_map_shadowObject_bmap;

sampler ObjShadowSampler = sampler_state
{
	Texture = <sc_map_shadowObject_bmap>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = CLAMP;
	AddressV = CLAMP;
};


texture sc_map_shadowPoint1_bmap;
sampler pointDepthSampler = sampler_state 
{ 
   Texture = <sc_map_shadowPoint1_bmap>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};


texture sc_map_aioEnvmap_bmap;

sampler testSampler = sampler_state 
{ 
   Texture = <sc_map_aioEnvmap_bmap>; 
   MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};


///////////////////////HELPER FUNCTIONS///////////////////////////////////////
// calculate the light attenuation factor
float DoLightFactor(float4 Light,float3 Pos)
{
   float fac = 0.f;
   if (Light.w > 0.f) {    
      float LD = length(Light.xyz-Pos)/Light.w;
      if (LD < 1.f)
         fac = saturate(1.f - LD);

   }
   return fac; // get the distance factor
}

// calculate the light attenuation factor on the front side
float DoLightFactor(float4 Light,float3 P,float3 N)
{
	float3 D = Light.xyz-P; // ray pointing from the light to the surface
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	
	if (NdotL >= 0.f) 
	   return saturate(NdotL*8)*DoLightFactor(Light,P);
	else
	   return 0.f;
}

float DoLightFactorN(float4 Light,float3 P,float3 N)
{
	float3 D = Light.xyz-P; // ray pointing from the light to the surface
	float NdotL = dot(N,normalize(D));   // angle between surface and light ray
	
	if (NdotL >= 0.f) 
	   return 2 * NdotL * DoLightFactor(Light,P);
	else
	   return 0.f;
}

float4 DoPointLight(float3 P, float3 N, float4 Light, float4 LightColor)
{
	return LightColor * DoLightFactorN(Light,P,N);
}

float4 DoLight(float3 P, float3 N, int i)
{
	
	if(vecLightPos[i].w < 100000) return DoPointLight(P,N,vecLightPos[i],vecLightColor[i]);
	else return 0;
}

float light_vsm_epsilon = 0.00003;
float light_shadow_bias = 0.0025;

float doVSM(float2 depthmap, float lightDepth)
{
     	//depthmap.y = (depthmap.x * depthmap.x);
     	float lit_factor = (lightDepth <= depthmap.x);
    	
    	// Variance shadow mapping
    	float E_x2 = depthmap.y;
    	float Ex_2 = depthmap.x * depthmap.x;
    	float variance = min(max(E_x2 - Ex_2, 0.0) + light_vsm_epsilon, 1.0);
    	float m_d = (depthmap.x - lightDepth);
    	float p = variance / (variance + m_d * m_d);
    	
    	// Adjust the light color based on the shadow attenuation
    	return 1-max(lit_factor, p);
}

float decode(float4 vec)
{
		const float4 bitShifts = float4(1.0/(256.0*256.0*256.0), 1.0/(256.0*256.0), 1.0/256.0, 1);

		return dot(vec.xyzw , bitShifts);
}

static const float SHADOW_EPSILON = 0.00005f;
float4 vecViewPort;

float PCF3x3(sampler shadowMap, float2 texC, float depth)
{    
    float2 texCoords[9];

	// Generate the tecture co-ordinates for the specified depth-map size
	// 4 3 5
	// 1 0 2
	// 7 6 8
	texCoords[0] = texC;
	texCoords[1] = texC + float2( -vecViewPort.z, 0.0f);
	texCoords[2] = texC + float2(  vecViewPort.z, 0.0f);
	texCoords[3] = texC + float2( 0.0f, -vecViewPort.z);
	texCoords[6] = texC + float2( 0.0f,  vecViewPort.z);
	texCoords[4] = texC + float2( -vecViewPort.z, -vecViewPort.z);
	texCoords[5] = texC + float2(  vecViewPort.z, -vecViewPort.z);
	texCoords[7] = texC + float2( -vecViewPort.z,  vecViewPort.z);
	texCoords[8] = texC + float2(  vecViewPort.z,  vecViewPort.z);
	
	// Sample each of them checking whether the pixel under test is shadowed or not
	float shadowCoeff = 0.0f;
	for( int i = 0; i < 9; i++ )
	{
		//is the texel shadowed?
		//shadowCoeff += (tex2D( shadowMap, texCoords[i] ).r + SHADOW_EPSILON) < depth ? 0.0f : 1.0f;
		shadowCoeff += (((decode(tex2D(ShadowMapSampler, texCoords[i]).wzyx))) + SHADOW_EPSILON) < depth ? 0.0f : 1.0f;
	}
	
	// Get the average, divide by 9
	shadowCoeff *= .1111111f;
	
	return shadowCoeff;
}

///////////////////////END OF HELPER FUNCTIONS///////////////////////////////////////



float3 sc_customLights_var[16];

float4 mainPS(float4 inTex:TEXCOORD0):COLOR0
{
	//get scene depth
	float depth = decode(tex2D(depthSampler,inTex.xy).wzyx);
	//depth = tex2D(depthSampler,inTex.xy).x;
	
	//compute world position
	float4 vp = float4(inTex.xy * float2(2, -2) - float2(1, -1), depth, 1);
   float4 v = mul(vp, matMtl);
   float3 position = v.xyz/v.w;
	
	//calculate normals from position
   float3 normal = 0;
   normal = cross(ddx(position),ddy(position));
   	
	//camera-to-surface vector
   float3 directionToCamera = normalize(vecViewPos.xyz - position);
    
    /*
   //lighting
	float3 Ln = (float3(-70,150,20)- position);
 	float attenuation = saturate(1.0f - length(Ln)/500);
   Ln = normalize(-vecSunDir);
	float3 Nb = normalize(normal);
   float3 Vn = directionToCamera;
   float3 Hn = normalize(Vn + Ln);
   float4 lighting = lit(dot(Ln,Nb),dot(Hn,Nb),15);
	*/
	
	//shadow mapping
	float4 ShadowMapSamplingPos = mul(float4(position,1), mul(matEffect3,matEffect1));
	float4 RealDistance = mul(float4(position,1), matEffect3).z/matEffect2[0].w;
	
	float4 OutShadow;
	OutShadow.x = ShadowMapSamplingPos.x;
	OutShadow.y = ShadowMapSamplingPos.y;
	OutShadow.z = ShadowMapSamplingPos.w;
	OutShadow.w = RealDistance.x;
	
	float2 ProjectedTexCoords;
   ProjectedTexCoords[0] = OutShadow.x/OutShadow.z/2.0f +0.5f;
   ProjectedTexCoords[1] = -OutShadow.y/OutShadow.z/2.0f +0.5f;
	
	float shadowDepth = tex2D(ShadowMapSampler, ProjectedTexCoords).x;
	float shadow = doVSM(float2(shadowDepth,shadowDepth*shadowDepth),RealDistance.x);
	shadow = (1-shadow);
	
   //float shadow = PCF3x3(ShadowMapSampler, ProjectedTexCoords, RealDistance.x);
   
   shadow *=(1-tex2D(ShadowAlphaSampler, ProjectedTexCoords));
   shadow += tex2D(ShadowAlphaSampler, ProjectedTexCoords);
   shadow = clamp(shadow,0,1);
	//--------------
	
	
	
	
	//lighting
	normal = normalize(normal);
	float4 color = 0;
	//color.rgb = clamp(dot(normal,normalize(-vecSunDir)),0,1)*shadow;
	//color.rgb = shadow+(1-clamp(dot(normal,normalize(-vecSunDir)),0,1));
	//color.rgb *= tex2D(sceneSampler,inTex).xyz;
	
	/*
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[0].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[0].xzy)- position))*float3(1,0,0);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[1].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[1].xzy)- position))*float3(0,1,0);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[2].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[2].xzy)- position))*float3(0,0,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[3].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[3].xzy)- position))*float3(0,1,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[4].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[4].xzy)- position))*float3(1,0,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[5].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[5].xzy)- position))*float3(0,0.5,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[6].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[6].xzy)- position))*float3(0.5,0,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[7].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[7].xzy)- position))*float3(0.5,0.5,0);
	
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[8].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[8].xzy)- position))*float3(1,0,0);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[9].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[9].xzy)- position))*float3(0,1,0);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[10].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[10].xzy)- position))*float3(0,0,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[11].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[11].xzy)- position))*float3(0,1,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[12].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[12].xzy)- position))*float3(1,0,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[13].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[13].xzy)- position))*float3(0,0.5,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[14].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[14].xzy)- position))*float3(0.5,0,1);
	color.rgb += clamp(1-distance(position,float3(sc_customLights_var[15].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[15].xzy)- position))*float3(0.5,0.5,0);
	*/
	
	
	//color.rgb = clamp(1-distance(position,float3(sc_customLights_var[0].xzy))/130,0,1)*dot(normal,normalize(float3(sc_customLights_var[0].xzy)- position))*float3(1,0,0);
	//color.rgb = clamp(color.rgb,0,1);
	//color.rgb = normalize(color.rgb);
	//color.rgb = 1;
	if(depth == 0) shadow = 1;




/*
	//per object shadows
	ShadowMapSamplingPos = mul(float4(position,1), matEffect4);
	//RealDistance = mul(float4(position,1), matEffect3).z/matEffect2[0].w;
	
	OutShadow;
	OutShadow.x = ShadowMapSamplingPos.x;
	OutShadow.y = ShadowMapSamplingPos.y;
	OutShadow.z = ShadowMapSamplingPos.w;
	OutShadow.w = RealDistance.x;
	
	ProjectedTexCoords;
   ProjectedTexCoords[0] = OutShadow.x/OutShadow.z/2.0f +0.5f;
   ProjectedTexCoords[1] = -OutShadow.y/OutShadow.z/2.0f +0.5f;

	shadow *= 1-tex2D(ObjShadowSampler, ProjectedTexCoords).xyz;
	*/
	/*
	// ONE POINT LIGHT (meh i dont have time, hurry up!)
	float pointDepth = distance(position,float3(288,96,-304))/(vecLightPos[0].w*1.5);
	float pdepth = decode(texCUBE(pointDepthSampler, -float4(float3(288,96,-304)-position, 1.0f)).wzyx);
  	float shadow1 = 1;
  	if(pdepth < pointDepth-0.015) shadow1 = 0;
	
	shadow1 *= pow((pointDepth),2);
	
	if(pointDepth < 0.99) pointDepth = 0;
	shadow1 = shadow1*(1-pointDepth)+pointDepth;
	
	float3 Ln = normalize(float3(288,96,-304) - position);
	float diff1 = clamp(dot(normal,-Ln),0,1);
	
	shadow1 = clamp(((shadow1+diff1)),0,1);
	//
	
	shadow *= shadow1;
	*/
	color.rgb = shadow+(1-clamp(dot(normal,normalize(-vecSunDir)),0,1));
	color.rgb = clamp(color.rgb,0,1);
	color.rgb *= tex2D(sceneSampler,inTex).rgb;
	
	
	return color;
}

technique t1
{
	pass p0
	{
		//zWriteEnable = true;
		//alphaTestEnable = true;
		PixelShader = compile ps_2_a mainPS();
	}
}