//------------------------------------------------------------------------------
// Untweakables
//------------------------------------------------------------------------------
float4x4 matWorldViewProj : WorldViewProjection;

float4x4 matWorldView : WorldView;

float4x4 matWorld : World;

float4x4 u_shadow_view;
float4x4 matView : View;

float4x4 u_shadow_view_inv;

float4x4 matProj : Projection;

float4x4 matParallelProj = {
	0.9941, 0, 0, 0,
	0, 1.3262, 0, 0,
	0, 0, 	1, 1,
	0, 0, -13000, 0
};

matrix matMtl;
matrix matEffect1;


//------------------------------------------------------------------------------
// Tweakables and Scene Graph Bindings
//------------------------------------------------------------------------------

float light_angle_atten_begin = 1.99;//: lightumbra;
float light_angle_atten_end = 550;//: lightpenumbra;
float light_atten_begin = 1.0;
float light_atten_end = 10000.0;
float light_shadow_bias = 0.0033;
float light_vsm_epsilon = 0.000001;

//higher values => brighter color/shadow
float ColorAmbient = 1; //Colormap brightness
float ShadowAmbient = 0.5; //Shadowmap brightness
//

float4 LightColor = {0.89,0.6235,0.229,1.0};

float2 g_vSampleOffsets[12] = {

   -0.00695914,  0.00457137,
   -0.00203345,  0.00620716,
    0.00962340, -0.00194983,
    0.00473434, -0.00480026,
    0.00519456,  0.00767022,
    0.00185461, -0.00893124,
    0.00507431,  0.00064425,
    0.00896420,  0.00412458,
   -0.00321940, -0.00932615,
   -0.00791559, -0.00597705,
   //
   -0.00326212, -0.00405805,
   -0.00840144, -0.0073580,

};


float vecSkill5;
float vecSkill9;
float vecSkill13;


float4 vecLightPos[8];

static float2 light_angle_atten = float2(light_angle_atten_begin, light_angle_atten_end) * 0.5;
static float2 cos_light_angle_atten = cos(light_angle_atten);

float3 diffuse_color : Diffuse = {1.0f, 1.0f, 1.0f};

//------------------------------------------------------------------------------
// Textures and Render Targets
//------------------------------------------------------------------------------

texture mtlSkin1; //depthmap
texture mtlSkin2; //colormap
texture mtlSkin3; //colormap


sampler light_shadow_map = sampler_state
{
    texture   = <mtlSkin2>;
    MinFilter = Anisotropic;
    MagFilter = Anisotropic;
    MipFilter = Linear;
		AddressU = Wrap;
		AddressV = Wrap;
};


sampler color_map = sampler_state
{
    texture   = <mtlSkin3>;
    MinFilter = Anisotropic;
    MagFilter = Anisotropic;
    MipFilter = Linear;
		AddressU = Wrap;
		AddressV = Wrap;
};


//------------------------------------------------------------------------------
// Lighting Pass Shaders
//------------------------------------------------------------------------------
struct LightingVSOutput
{
	float4 position_screen : POSITION;
	float3 position_world  : TEXCOORD0;
	float3 normal_world    : TEXCOORD1;
	float3 light_position  : TEXCOORD2;
	float3 light_direction : TEXCOORD3;
	float3 TexCoords : TEXCOORD4;
};

LightingVSOutput lighting_VS(
		float4 position : POSITION,
		float3 normal   : NORMAL,
		uniform float4x4 tshadow)
{      
   LightingVSOutput output = (LightingVSOutput)0;
   
   /*
   //--------------------------------------------
   //calculate ViewInv
   //find an other way to do this! 
   //don't do this in the shader! 
   tshadow[0].x = matMtl[0].x;
  	tshadow[0].y = matMtl[1].x;
  	tshadow[0].z = matMtl[2].x;
  	tshadow[0].w = matMtl[0].w;
  	
  	tshadow[1].x = matMtl[0].y;
  	tshadow[1].y = matMtl[1].y;
  	tshadow[1].z = matMtl[2].y;
  	tshadow[1].w = matMtl[1].w;
  	
  	tshadow[2].x = matMtl[0].z;
  	tshadow[2].y = matMtl[1].z;
  	tshadow[2].z = matMtl[2].z;
  	tshadow[2].w = matMtl[2].w;
        
   tshadow[3].x = vecSkill5;
  	tshadow[3].y = vecSkill9;
  	tshadow[3].z = vecSkill13;
  	tshadow[3].w = matMtl[3].w;
   //-------------------------------------------
     */
     
    output.position_screen  = mul(position, matWorldViewProj);
    output.position_world   = mul(position, matWorld).xyz;
    output.normal_world     = mul(float4(normal, 0), matWorld).xyz;
    
    // Work out the light position and direction in world space
    output.light_position   = float3(tshadow._41, tshadow._42, tshadow._43);
    output.light_direction  = float3(tshadow._31, tshadow._32, tshadow._33);
        
    //colormap texcoords
    output.TexCoords = mul(position, matWorldView).xyz;
    
    
    return output;
}

struct LightingPSOutput
{
	float4 color : COLOR;
};

LightingPSOutput lighting_PS(
		LightingVSOutput input,
		uniform float4x4 shadow_view_projection)
{
	LightingPSOutput output = (LightingPSOutput)0;

	shadow_view_projection[1].w = 0;

	// Renormalize
    float3 normal_world = normalize(input.normal_world);
    float3 light_direction = normalize(input.light_direction);

    // Sum the contributions from all lights
    float3 lit_color = float3(0, 0, 0);
      
  	// Light Shader:
  	float3 light_contrib;
  	float3 dir_to_light;
  	float  dist_to_light;
  	float  n_dot_l;
  	 	
  	{  		  	
    	// Unnormalized light vector
    	dir_to_light = input.light_position - input.position_world;
    	dist_to_light = length(dir_to_light);
		
		/*
		float atten_amount =
			clamp((dist_to_light   - light_atten_begin) /
        	      (light_atten_end - light_atten_begin),
        	       0.0, 1.0);
		*/
    	// Radial attenuation
    	dir_to_light = normalize(dir_to_light);
    	//float2 cos_angle_atten = cos_light_angle_atten;
    	//float  cos_angle = dot(-dir_to_light, light_direction);
    	
    	/*
    	float angle_atten_amount = 
        	clamp((cos_angle         - cos_angle_atten.x) /
                  (cos_angle_atten.y - cos_angle_atten.x),
            	   0.0, 1.0);
        */    	   
    	// Compose the light shader outputs
    	//light_contrib = (1.0 - atten_amount) * (1.0 - angle_atten_amount) * LightColor;
    	n_dot_l       = dot(normal_world, dir_to_light);
  	}
  	
  	light_contrib = (1,1,1);
  	// Variance Shadow Mapping:
  	{
  		// Transform the surface into light space and project
  		// NB: Could be done in the vertex shader, but doing it here keeps the "light
  		// shader" abstraction and doesn't limit # of shadowed lights.

  		float4 surf_tex = mul(float4(input.position_world, 1.0), shadow_view_projection);
    	surf_tex = surf_tex / surf_tex.w;
    	
    	// Rescale viewport to be [0,1] (texture coordinate space)
    	float2 shadow_tex = surf_tex.xy * float2(0.5, -0.5) + 0.5;
	
    	float4 moments;
    	// TODO: Emulate bilinear filtering on unsupporting hardware
      	moments = tex2D(light_shadow_map, shadow_tex);
      	//moments += tex2D(light_shadow_map2, shadow_tex + 0.001);
      
      	for(int i = 0; i < 7; i++ )
   		{
   			moments += tex2D( light_shadow_map, shadow_tex + g_vSampleOffsets[i]/6);
	   	}
   
   		moments /= 8;
   		
 	
    	// Rescale light distance and check if we're in shadow
    	float rescaled_dist_to_light = dist_to_light / light_atten_end;
    	rescaled_dist_to_light -= light_shadow_bias;
    	float lit_factor = (rescaled_dist_to_light <= moments.x);
    	
    	// Variance shadow mapping
    	float E_x2 = (moments.x)/6000;
    	float Ex_2 = (moments.y * moments.y)/90000;
    	float variance = min(max(E_x2 - Ex_2, 0.0) + (light_vsm_epsilon), 1.0);
    	float m_d = (moments.x - rescaled_dist_to_light);
    	float p = variance / (variance + m_d * m_d);
    	
    	// Adjust the light color based on the shadow attenuation
    	light_contrib *= max(lit_factor, p);
  	}
     
      
  	// Evaluate basic diffuse lighting
  	float self_shadow = clamp(n_dot_l * 2.0, 0.0, 1.0);
  	float3 direct_contrib = diffuse_color * n_dot_l;
  	
  	lit_color += (light_contrib) * self_shadow * direct_contrib * LightColor;

   //output.color = float4(lit_color, 1.0)+ShadowAmbient;
   
   //colormap
   float2 tPos = input.TexCoords.xy/input.TexCoords.z;

//arc60   
//tPos.y = 1 - (tPos.y*1.1449 + 0.5);
//tPos.x = (tPos.x*0.865 + 0.5);   
	tPos.y = 1 - (tPos.y*0.658 + 0.5);
	tPos.x = tPos.x*0.499 + 0.5;
	//output.color *= tex2D(color_map, tPos.xy)*ColorAmbient;

	output.color = (float4(lit_color, 1.0)+ShadowAmbient)*(tex2D(color_map, tPos.xy)*ColorAmbient); 
	//output.color.rgb /= 2;  
 
   return output;
}

//------------------------------------------------------------------------------
// Techniques
//------------------------------------------------------------------------------

technique Main
{
	
	pass UseShadowMap
	{
		VertexShader = compile vs_1_0 lighting_VS(matEffect1); //u_shadow_view_inv
		ZEnable = true;
		ZWriteEnable = true;
		ZFunc = LessEqual;
		CullMode = None;
		PixelShader = compile ps_2_0 lighting_PS(mul(matMtl, matProj)); //u_shadow_view
	}
	
}